#include "sdl.h"
#include "sdl_mixer.h"
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <ctime>

// Factor de conversion grados a radianes:
static constexpr float pi= M_PI;
static constexpr float factor= pi/180.0;
static constexpr float probabilidad = 50.0; // 50%
static constexpr int intervalo = 15000; // 15000 15"

// Gráfico de nave, 5 puntos, 4 segmentos:
static const std::vector<sdl::FPoint> nave = { {15.0, 0.0}, {-15.0, -9.0}, {-11.0, 0.0}, {-15.0, 9.0}, {15.0, 0.0} };
static const std::vector<sdl::FPoint> estela = { {-16.0, -4.0}, {-25.0, 0.0}, {-16.0, 4.0} };
static const std::vector<sdl::Shape> formanave { sdl::FCircle(-6.0, 0.0, 9.0), sdl::FCircle(8.0, 0.0, 5.0) };

static const std::vector<std::vector<sdl::FPoint>> asteroide = {
    {{-5.0, -18.0}, {10.0, -16.0}, {19.0, -3.0}, {10.0, 7.0}, {11.0, 15.0}, {-5.0, 19.0}, {-18.0,8.0}, {-14.0, -7.0}, {-5.0, -18.0} },
    {{-10.0, -14.0}, {2.0, -14.0}, {11.0, 1.0}, {14.0, 1.0}, {8.0, 9.0}, {-2.0, 14.0}, {-9.0,6.0}, {-14.0, 4.0}, {-9.0, -5.0}, {-10.0,-14.0} },
    {{-4.0, -10.0}, {2.0, -10.0}, {9.0, -6.0}, {5.0, 1.0}, {8.0, 5.0}, {0.0, 9.0}, {-5.0,-2.0}, {-7.0, -4.0}, {-4.0, -10.0} }};
static const std::vector<std::vector<sdl::Shape>> forma = {
    { sdl::FCircle(0, 0, 19) },
    { sdl::FCircle(0, 0, 14) },
    { sdl::FCircle(0, 0, 10) }
};
static const std::vector<sdl::FPoint> disparo = { {-1.0, 0.0}, {0.0, 1.0}, {1.0, 0.0}, {0.0, -1.0}, {-1.0, 0.0} };
static const std::vector<sdl::Shape> formadisparo { sdl::FCircle(0.0, 0.0 ,1.0) };

static const std::vector<sdl::FPoint> platilloA = {
    {-10.0,-4.0}, {-8.0, -8.0}, {7.0,-8.0}, {9.0,-4.0}, {15.0,2.0},
    {9.0,8.0}, {-10.0,8.0}, {-16.0,2.0}, {-10.0,-4.0}, {9.0, -4.0} };
static const std::vector<sdl::FPoint> platilloB = { {-16.0,2.0}, {15.0, 2.0} };
static const std::vector<sdl::Shape> formaplatillo = {
    sdl::FRect(-9.0, -8.0, 18.0, 17.0),
    sdl::FCircle(-9.0, 2.0, 6.0),
    sdl::FCircle(8.0, 2.0, 6.0) };

static const std::vector<sdl::FPoint> platilloAp = {
    {-7.0,-3.0}, {-5.0, -5.0}, {5.0,-5.0}, {6.0,-3.0}, {10.0,1.0},
    {6.0,5.0}, {-7.0,5.0}, {-11.0,1.0}, {-7.0,-3.0}, {6.0, -3.0} };
static const std::vector<sdl::FPoint> platilloBp = { {-11.0,1.0}, {10.0, 1.0} };
static const std::vector<sdl::Shape> formaplatillop = {
    sdl::FRect(-6.0,-5.0,12.0, 12.0),
    sdl::FCircle(-6.0, 1.0, 4.0),
    sdl::FCircle(5.0, 1.0, 4.0) };

enum tipoAsteroide {
    grande=0,
    mediano,
    pequeno,
    nada
};
enum tipoPlatillo {
    pgrande=0,
    ppequeno
};

class Platillo {
private:
    std::vector<sdl::FPoint> platilloA2;
    std::vector<sdl::FPoint> platilloB2;
    sdl::Hitbox hitbox;
    sdl::FPoint posicion;
    sdl::FPoint velocidad;
    tipoPlatillo tipo;
    bool activo;
    bool primero; // Para establecer una semilla srand la primera vez
    bool salir;   // Condicion para terminar el hilo
    void trasladar(sdl::FPoint &desplazamiento, sdl::FPoint &escala) {
        for(size_t i = 0; i < platilloA.size(); i++) {
            if(tipo == pgrande)
                platilloA2[i] = platilloA[i].Move(0, desplazamiento, escala);
            else
                platilloA2[i] = platilloAp[i].Move(0, desplazamiento, escala);
        }
        for(size_t i = 0; i < platilloB.size(); i++) {
            if(tipo == pgrande)
                platilloB2[i] = platilloB[i].Move(0, desplazamiento, escala);
            else
                platilloB2[i] = platilloBp[i].Move(0, desplazamiento, escala);
        }
    }

public:
    Platillo(const sdl::FPoint &pos, const sdl::FPoint &vel) : posicion(pos), velocidad(vel), activo(false), primero(true), salir(false) {
        platilloA2.resize(platilloA.size());
        platilloB2.resize(platilloB.size());
        for(size_t i=0; i<formaplatillo.size(); i++) hitbox.InsertShape(formaplatillo[i]);
        hitbox.ResizeT();
    }
    void Actualizar(Uint64 tick, sdl::FPoint escala) {
        posicion.X() += (velocidad.X()*(tick)/1000.0);
        posicion.Y() += (velocidad.Y()*(tick)/1000.0);
        if(posicion.X()>635) posicion.X() -=640;
        if(posicion.X()<5) posicion.X() +=640;
        if(posicion.Y()>475) posicion.Y() -=480;
        if(posicion.Y()<5) posicion.Y() +=480;

        // Trasladar platillo:
        trasladar(posicion, escala);
        // Trasladar hitbox:
        hitbox.Move(0, posicion, escala);
    }
    std::vector<sdl::FPoint> &PuntosPlatilloA() { return platilloA2; }
    std::vector<sdl::FPoint> &PuntosPlatilloB() { return platilloB2; }
    void Activar();
    void Desactivar() { activo=false; }
    bool Activo()  { return activo; }
    bool EsPrimero() { return primero; }
    void Primero(bool p) { primero = p; }
    bool &Salir() { return salir; }
    tipoPlatillo Tipo() { return tipo; }
    sdl::FPoint &Posicion() { return posicion; }
    sdl::FPoint &Velocidad() { return velocidad; }
    sdl::Hitbox &Hitbox() { return hitbox; }
};

void Platillo::Activar() {
    float angulo = sdl::random(0.5*pi);
    int cuadrante = SDL_floor(sdl::random(4.0));
    tipo = static_cast<tipoPlatillo>(SDL_floor(sdl::random(2)));
    hitbox.Reset();
    if(tipo == pgrande)
        for(size_t i=0; i<formaplatillop.size(); i++) hitbox.InsertShape(formaplatillo[i]);
    else
        for(size_t i=0; i<formaplatillop.size(); i++) hitbox.InsertShape(formaplatillop[i]);

    switch(cuadrante) {
        case 0:
        posicion = {20.0, 20.0};
        velocidad = { 100*SDL_cosf(angulo), 100*SDL_sinf(angulo) };
        break;
        case 1:
        posicion = {20.0, 460.0};
        velocidad = { 100*SDL_cosf(angulo), -100*SDL_sinf(angulo) };
        break;
        case 3:
        posicion = {620.0, 20.0};
        velocidad = { -100*SDL_cosf(angulo), 100*SDL_sinf(angulo) };
        break;
        default:
        posicion = {620.0, 460.0};
        velocidad = { -100*SDL_cosf(angulo), -100*SDL_sinf(angulo) };
        break;
    }
    activo = true;
}

class Nave {
private:
    std::vector<sdl::FPoint> nave2;
    std::vector<sdl::FPoint> estela2;
    sdl::Hitbox hitbox;
    sdl::FPoint posicion;
    sdl::FPoint velocidad;
    void trasladar(float alfa, sdl::FPoint &desplazamiento, sdl::FPoint &escala) {
        for(size_t i = 0; i < nave.size(); i++) {
            nave2[i] = nave[i].Move(alfa, desplazamiento, escala);
        }
        for(size_t i = 0; i < estela.size(); i++) {
            estela2[i] = estela[i].Move(alfa, desplazamiento, escala);
        }
    }

public:
    Nave(const sdl::FPoint &pos, const float alfa) : posicion(pos), velocidad(0.0, 0.0) {
        nave2.resize(nave.size());
        estela2.resize(estela.size());
        for(size_t i=0; i<formanave.size(); i++) hitbox.InsertShape(formanave[i]);
        hitbox.ResizeT();
    }
    void Actualizar(Uint64 tick, float acelerar, float alfa, sdl::FPoint escala) {
        velocidad.X() += acelerar*cos(factor*alfa);
        velocidad.Y() += -acelerar*sin(factor*alfa);
        posicion.X() += (velocidad.X()*(tick)/1000.0);
        posicion.Y() += (velocidad.Y()*(tick)/1000.0);
        if(posicion.X()>635) posicion.X() -=640;
        if(posicion.X()<5) posicion.X() +=640;
        if(posicion.Y()>475) posicion.Y() -=480;
        if(posicion.Y()<5) posicion.Y() +=480;
        // Trasladar nave:
        trasladar(factor*alfa, posicion, escala);
        // Trasladar hitbox:
        hitbox.Move(factor*alfa, posicion, escala);
    }
    std::vector<sdl::FPoint> &PuntosNave() { return nave2; }
    std::vector<sdl::FPoint> &PuntosEstela() { return estela2; }
    sdl::FPoint &Posicion() { return posicion; }
    sdl::FPoint &Velocidad() { return velocidad; }
    sdl::Hitbox &Hitbox() { return hitbox; }
};

class Asteroide {
private:
    std::vector<sdl::FPoint> asteroide2;
    tipoAsteroide tipo;
    sdl::FPoint posicion;
    sdl::FPoint velocidad;
    float alfa; // En radianes
    float vRotacion; // En radianes/tick
    sdl::Hitbox hitbox;
    bool activo;
    void Trasladar(float alfa, sdl::FPoint &desplazamiento, sdl::FPoint &escala) {
    // x' = x * x" * cos(α) + y * y" * sin(α) + dx
    // y' = -x * x" * sin(α) + y * y" * cos(α) + dy
        for(size_t i = 0; i < asteroide2.size(); i++) {
            asteroide2[i] = asteroide[tipo][i].Move(alfa, desplazamiento, escala);
        }
    }
public:
    Asteroide(const sdl::FPoint &pos, tipoAsteroide t) : tipo(t), posicion(pos),
        //alfa(360.0*std::rand()/RAND_MAX), activo(true) {
        alfa(sdl::random(2.0*pi)), activo(true) {
        asteroide2.resize(asteroide[tipo].size());
        //float v = 100.0*std::rand()/RAND_MAX;
        float v = sdl::random(100.0);
        //alfa = 2.0*pi*std::rand()/RAND_MAX;
        velocidad = {v*(float)cos(alfa), v*(float)sin(alfa)};
        //vRotacion = -5.0+10.0*std::rand()/RAND_MAX;
        vRotacion = -5.0+sdl::random(10.0);
        for(size_t i=0; i<forma[tipo].size(); i++) hitbox.InsertShape(forma[tipo][i]);
        hitbox.ResizeT();
    }
    Asteroide(const sdl::FPoint &pos, tipoAsteroide t, const sdl::FPoint &vel) : tipo(t), posicion(pos), velocidad(vel) {
        //vRotacion = -5.0+10.0*std::rand()/RAND_MAX;
        vRotacion = -5.0+sdl::random(10.0);
        for(size_t i=0; i<forma[tipo].size(); i++) hitbox.InsertShape(forma[tipo][i]);
        asteroide2.resize(asteroide[tipo].size());
        hitbox.ResizeT();
        }
    void Actualizar(Uint64 tick, sdl::FPoint escala) {
        posicion.X() += (velocidad.X()*(tick)/1000.0);
        posicion.Y() += (velocidad.Y()*(tick)/1000.0);
        alfa += vRotacion *(tick)/1000.0;
        if(alfa > 2*pi) alfa -= 2*pi;
        if(alfa < 0) alfa += 2*pi;
        if(posicion.X()>645) posicion.X() -=650;
        if(posicion.X()<-5) posicion.X() +=650;
        if(posicion.Y()>485) posicion.Y() -=490;
        if(posicion.Y()<-5) posicion.Y() +=490;
        // Trasladar asteroide:
        Trasladar(alfa, posicion, escala);
        // Trasladar hitbox:
        hitbox.Move(alfa, posicion, escala);
    }
    tipoAsteroide &Tipo() { return tipo; }
    void CambiarTipo(tipoAsteroide tipo);
    sdl::FPoint &Posicion() { return posicion; }
    sdl::FPoint &Velocidad() { return velocidad; }
    bool Colision(Asteroide &a) {
        return hitbox.Collision(a.hitbox);
    }
    sdl::Hitbox &Hitbox() { return hitbox; }
    std::vector<sdl::FPoint> &Puntos() { return asteroide2; }
    bool Activo() { return activo; }
    void Desactivar() { activo=false; }
};

void Asteroide::CambiarTipo(tipoAsteroide t) {
    tipo = t;
    //vRotacion = -5.0+10.0*std::rand()/RAND_MAX;
    vRotacion = -5.0+sdl::random(10.0);
    hitbox.Reset();
    for(size_t i=0; i<forma[tipo].size(); i++) hitbox.InsertShape(forma[tipo][i]);
    asteroide2.resize(asteroide[tipo].size());
    hitbox.ResizeT();
}

class Disparo {
private:
    std::vector<sdl::FPoint> disparo2;
    sdl::FPoint posicion;
    sdl::FPoint velocidad;
    sdl::Hitbox hitbox;
    float ticksvida;
    bool activo;

    void Trasladar(sdl::FPoint &desplazamiento, sdl::FPoint &escala) {
    // x' = x * x" * cos(α) + y * y" * sin(α) + dx
    // y' = -x * x" * sin(α) + y * y" * cos(α) + dy
        for(size_t i = 0; i < disparo2.size(); i++) {
            disparo2[i] = disparo[i].Move(0.0, desplazamiento, escala);
        }
    }

public:
    Disparo() : activo(false) {}
    // Sumamos 100.0 a la velocidad de la nave:
    Disparo(sdl::FPoint pos, float alfa, sdl::FPoint v) : posicion(pos),
        velocidad(v.X()+150*SDL_cosf(factor*alfa), v.Y()-150*SDL_sinf(factor*alfa)), ticksvida(2000), activo(true) {
        disparo2.resize(disparo.size());
        for(size_t i=0; i<formadisparo.size(); i++) hitbox.InsertShape(formadisparo[i]);
        hitbox.ResizeT();
    }
    void Actualizar(Uint64 tick, sdl::FPoint escala) {
        ticksvida -= tick;
        if(ticksvida < 0) activo=false;
        else {
            posicion.X() += (velocidad.X()*(tick)/1000.0);
            posicion.Y() += (velocidad.Y()*(tick)/1000.0);
            if(posicion.X()>645) posicion.X() -=650;
            if(posicion.X()<-5) posicion.X() +=650;
            if(posicion.Y()>485) posicion.Y() -=490;
            if(posicion.Y()<-5) posicion.Y() +=490;
            // Trasladar asteroide:
            Trasladar(posicion, escala);
            // Trasladar hitbox:
            hitbox.Move(0.0, posicion, escala);
        }
    }
    bool Activo() { return activo; }
    void Desactivar() { activo=false; }
    sdl::FPoint &Posicion() { return posicion; }
    sdl::FPoint &Velocidad() { return velocidad; }
    std::vector<sdl::FPoint> &Puntos() { return disparo2; }
    bool Colision(Asteroide &a) {
        return hitbox.Collision(a.Hitbox());
    }
    bool Colision(Platillo &p) {
        return hitbox.Collision(p.Hitbox());
    }
    bool Colision(Nave &n) {
        return hitbox.Collision(n.Hitbox());
    }
};

Uint32 cbPlatillo(Uint32 intervalo, void* param) {
    Platillo *platillo = static_cast<Platillo*>(param);
    if(platillo->EsPrimero()) { // Establecer una semilla rand la primera vez
        std::srand(std::time(NULL));
        platillo->Primero(false);
    }
    if(sdl::random(100)<probabilidad) {
        platillo->Activar();
        return 0;
    }
    return intervalo;
}

int hiloPlatillo(void *param) {
    Uint64 t=SDL_GetTicks64();
    Platillo *platillo = static_cast<Platillo*>(param);
    float alfa;
    sdl::FPoint v2;

    while(!platillo->Salir()) {
        // Cada segundo:
        if(SDL_GetTicks64()-t > 1000) {
// x' =  x" * cos(α) + y" * sin(α)
// y' = -x" * sin(α) + y" * cos(α)
            alfa = sdl::random(pi/2.0)-(pi/4.0);
            v2.X() =  platillo->Velocidad().X()*SDL_cosf(alfa)+platillo->Velocidad().Y()*SDL_sinf(alfa);
            v2.Y() = -platillo->Velocidad().X()*SDL_sinf(alfa)+platillo->Velocidad().Y()*SDL_cosf(alfa);
            platillo->Velocidad() = v2;
            t = SDL_GetTicks64();
        }
    }
    return 0;
}

class Asteroides : public sdl::Game, sdl::Mixer {
private:
    sdl::FPoint escala; // Escala del juego (ajustada a una resolución de 640x480
    std::vector<sdl::FPoint> estrellas;
    Nave nave;
    std::vector<Disparo> disparos;
    std::vector<Asteroide> asteroides;
    Platillo platillo;
    Disparo disparoP;
    sdl::Timer tPlatillo;
    sdl::Thread tHilo;
    float acelerar;
    float alfa;
    bool disparo;
    void SonidoImpactoAsteroide(sdl::FPoint &pos);
    void RebotarAsteroides(int a, int b);
    void DividirAsteroide(int iAst, float masa, sdl::FPoint &pos, sdl::FPoint &vel);
    void DestruirNave() { explosion[1].Play(); }
    void DestruirPlatillo();
    bool ColisionNave(int iAsteroide) { return asteroides[iAsteroide].Hitbox().Collision(nave.Hitbox()); }
    bool ColisionPlatillo(int iAsteroide) { return asteroides[iAsteroide].Hitbox().Collision(platillo.Hitbox()); }
    bool ColisionNavePlatillo() { return nave.Hitbox().Collision(platillo.Hitbox()); }
    sdl::Music musica;
    sdl::Chunk explosion[3];
    sdl::Chunk laser;

public:
    Asteroides(sdl::Window& window, sdl::Renderer& renderer) :
      sdl::Game(window, renderer, 800, 600),
      sdl::Mixer(MIX_INIT_MID|MIX_INIT_OGG),
      nave({320.0, 240.0}, 0.0),
      platillo({320.0,180.0}, {4.0,4.0}),
      acelerar(0.0), alfa(0.0), disparo(false),
      musica("asi_hablo_zaratustra_amanecer.mid"),
      explosion {"explosion1.ogg", "explosion2.ogg", "explosion3.ogg"},
      laser("laser3.mp3") {
        static int w, h;
        window.GetSize(&w, &h);
        escala = sdl::FPoint(w/640.0, h/480.0);
        musica.Play(0);
        for(int i=0; i<500; i++) {
            estrellas.push_back({sdl::randomf(w), sdl::randomf(h)});
        }
        tPlatillo.SetCallback(cbPlatillo);
        tPlatillo.SetInterval(intervalo);
        tPlatillo.SetParam(&platillo);
        tPlatillo.Init();
        tHilo.SetFunction(hiloPlatillo);
        tHilo.SetData(&platillo);
        tHilo.Init("HiloPlatillo");
    }
    ~Asteroides() { platillo.Salir() = true; } // Terminar el hilo
    const sdl::FPoint &Escala() { return escala; }
    void Init();
    void Events();
    void Update();
    void Render();
};

void Asteroides::Events() {
    static bool izq;
    static bool der;
    static bool arr;
    static bool dis;

    while(event.Poll()) {
        switch(event.Type()) {
        case SDL_KEYDOWN:
            if(event.Key().keysym.sym == SDLK_LEFT)  { izq = true; }
            if(event.Key().keysym.sym == SDLK_RIGHT) { der = true; }
            if(event.Key().keysym.sym == SDLK_UP)    { arr = true; }
            if(event.Key().keysym.sym == SDLK_SPACE && !dis) { dis = disparo = true; }
            if(event.Key().keysym.sym == SDLK_ESCAPE) { quit = true; }
            break;
        case SDL_KEYUP:
            if(event.Key().keysym.sym == SDLK_LEFT)  { izq = false; }
            if(event.Key().keysym.sym == SDLK_RIGHT) { der = false; }
            if(event.Key().keysym.sym == SDLK_UP)    { arr = false; }
            if(event.Key().keysym.sym == SDLK_SPACE) { dis = false; }
            break;
        }
    }
    acelerar = 0.0;
    if(izq && !der) alfa -= tick/10.0;
    if(!izq && der) alfa += tick/10.0;
    if(alfa < 0) alfa+=360.0;
    if(alfa >= 360.0) alfa-=360.0;
    if(arr) acelerar = tick/15.0;
}

void Asteroides::Render() {
    sdl::FRect re;
    sdl::Color negro("#000000ff");
    sdl::Color blanco("#ffffffff");
    sdl::Color naranja("#ffff00ff");
    sdl::Color rojo("#ff0000ff");
    sdl::Color verde("#00ff00ff");
    sdl::Color azul1("#4040ffff");
    sdl::Color azul2("#6060ffff");

    // Renderizado:
    renderer.SetDrawColor(negro);
    renderer.Clear();
    renderer.SetDrawColor(blanco);
    for(size_t i=0; i<estrellas.size(); i++)
        renderer.DrawPoint(estrellas[i]);
    renderer.DrawLinesF(nave.PuntosNave());
    //renderer.SetDrawColor(96, 96, 255, 255);
//#define VERHITBOX
#ifdef VERHITBOX
    re=sdl::FRect(nave.Hitbox().GetForma(0).Ci().X()-nave.Hitbox().GetForma(0).Ci().R(),
        nave.Hitbox().GetForma(0).Ci().Y()-nave.Hitbox().GetForma(0).Ci().R(),
        nave.Hitbox().GetForma(0).Ci().R()*2, nave.Hitbox().GetForma(0).Ci().R()*2);
    renderer.DrawRect(re);
    re=sdl::FRect(nave.Hitbox().GetForma(1).Ci().X()-nave.Hitbox().GetForma(1).Ci().R(),
        nave.Hitbox().GetForma(1).Ci().Y()-nave.Hitbox().GetForma(1).Ci().R(),
        nave.Hitbox().GetForma(1).Ci().R()*2, nave.Hitbox().GetForma(1).Ci().R()*2);
    renderer.DrawRect(re);
    for(size_t i=0; i<asteroides.size(); i++) {
        if(asteroides[i].Activo()) {
            re=sdl::FRect(asteroides[i].Hitbox().GetForma(0).Ci().X()-asteroides[i].Hitbox().GetForma(0).Ci().R(),
                asteroides[i].Hitbox().GetForma(0).Ci().Y()-asteroides[i].Hitbox().GetForma(0).Ci().R(),
                asteroides[i].Hitbox().GetForma(0).Ci().R()*2, asteroides[i].Hitbox().GetForma(0).Ci().R()*2);
            renderer.DrawRect(re);
        }
    }
    if(platillo.Activo()) {
        renderer.DrawRect(platillo.Hitbox().GetForma(0).Re());
        re=sdl::FRect(platillo.Hitbox().GetForma(1).Ci().X()-platillo.Hitbox().GetForma(1).Ci().R(),
            platillo.Hitbox().GetForma(1).Ci().Y()-platillo.Hitbox().GetForma(1).Ci().R(),
            platillo.Hitbox().GetForma(1).Ci().R()*2, platillo.Hitbox().GetForma(1).Ci().R()*2);
        renderer.DrawRect(re);
        re=sdl::FRect(platillo.Hitbox().GetForma(2).Ci().X()-platillo.Hitbox().GetForma(2).Ci().R(),
            platillo.Hitbox().GetForma(2).Ci().Y()-platillo.Hitbox().GetForma(2).Ci().R(),
            platillo.Hitbox().GetForma(2).Ci().R()*2, platillo.Hitbox().GetForma(2).Ci().R()*2);
        renderer.DrawRect(re);
    }

#endif // VERHITBOX
    if(acelerar != 0.0) {
        renderer.SetDrawColor(naranja);
        renderer.DrawLinesF(nave.PuntosEstela());
    }
    // Disparos:
    renderer.SetDrawColor(rojo);
    for(size_t i=0; i<disparos.size(); i++)
        if(disparos[i].Activo())
            renderer.DrawLinesF(disparos[i].Puntos());
    renderer.SetDrawColor(naranja);
    if(disparoP.Activo())
        renderer.DrawLinesF(disparoP.Puntos());

    // Asteroides:
    renderer.SetDrawColor(verde);
    for(size_t i=0; i<asteroides.size(); i++) {
        if(asteroides[i].Activo()) {
            renderer.DrawLinesF(asteroides[i].Puntos());
        }
    }
    if(platillo.Activo()) {
        renderer.SetDrawColor(azul1);
        renderer.DrawLinesF(platillo.PuntosPlatilloA());
        renderer.SetDrawColor(azul2);
        renderer.DrawLinesF(platillo.PuntosPlatilloB());
    }
    renderer.Present();
}

void Asteroides::Update() {
    bool insertado = false;

    // Actualizar objetos: nave, asteroides, disparos y platillo
    nave.Actualizar(tick, acelerar, alfa, escala);
    for(size_t i=0; i<asteroides.size(); i++) asteroides[i].Actualizar(tick, escala);
    if(disparoP.Activo()) disparoP.Actualizar(tick, escala);
    if(platillo.Activo()) {
        platillo.Actualizar(tick, escala);
    }
    for(size_t i=0; i<disparos.size(); i++) disparos[i].Actualizar(tick, escala);

    // Detectar colisiones entre asteroides, con nave y con platillo
    for(size_t i=0; i<asteroides.size()-1; i++) {
        if(asteroides[i].Activo()) {
            if(ColisionNave(i)) {
                DestruirNave();
                DividirAsteroide(i, 12.0, nave.Posicion(), nave.Velocidad());
            } else
            if(platillo.Activo() && ColisionPlatillo(i)) {
                DestruirPlatillo();
                SonidoImpactoAsteroide(platillo.Posicion());
                DividirAsteroide(i, 10.0, nave.Posicion(), nave.Velocidad()); // Masa de platillo en funcion de tipo
            } else
            for(size_t j=i+1; j<asteroides.size(); j++) {
                if(asteroides[j].Activo() && asteroides[i].Colision(asteroides[j])) {
                    RebotarAsteroides(i, j);
                }
            }
        }
    }

    // Detectar colision entre nave y platillo
    if(platillo.Activo() && ColisionNavePlatillo()) {
        DestruirNave();
        platillo.Desactivar();
        tPlatillo.Init();
    }

    // Detectar colisiones de disparos con platillo y asteroides
    for(size_t i=0; i<disparos.size(); i++) {
        if(disparos[i].Activo()) {
            // Verificar impacto disparos y platillo
            if(platillo.Activo() && disparos[i].Colision(platillo)) {
                SonidoImpactoAsteroide(disparos[i].Posicion()); // Sonido en función de posicion y distancia
                disparos[i].Desactivar();
                DestruirPlatillo();
            }
            // Verificar impacto disparos y asteroides
            for(size_t j=0; j<asteroides.size(); j++) {
                if(asteroides[j].Activo() && disparos[i].Colision(asteroides[j])) {
                    DividirAsteroide(j, 8.0, disparos[i].Posicion(), disparos[i].Velocidad());
                    SonidoImpactoAsteroide(disparos[i].Posicion());
                    disparos[i].Desactivar();
                }
            }
        }
    }

    // Detectar colisiones de disparo de platillo con nave y asteroides
    if(disparoP.Activo()) {
        if(disparoP.Colision(nave)) {
            DestruirNave();
            disparoP.Desactivar();
        } else
        for(size_t j=0; j<asteroides.size(); j++) {
            if(asteroides[j].Activo() && disparoP.Colision(asteroides[j])) {
                DividirAsteroide(j, 8.0, disparoP.Posicion(), disparoP.Velocidad());
                SonidoImpactoAsteroide(disparoP.Posicion());
                disparoP.Desactivar();
            }
        }
    } else {
        if(platillo.Activo()) {
            disparoP = Disparo(platillo.Posicion(), sdl::random(360.0), platillo.Velocidad());
            laser.Play(1); // modificar en funcion de distancia
        }
    }

    if(disparo) {
        // Buscar un disparo inactivo:
        for(size_t i=0; i<disparos.size() && !insertado; i++) {
            if(!disparos[i].Activo()) {
                disparos[i] = Disparo(nave.Posicion(),alfa, nave.Velocidad());
                insertado = true;
            }
        }
        if(!insertado) disparos.push_back(Disparo(nave.Posicion(), alfa, nave.Velocidad()));
        disparo=false;
        laser.Play(1);
    }
}

void Asteroides::DestruirPlatillo() {
    platillo.Desactivar();
    tPlatillo.Init();
}

void Asteroides::SonidoImpactoAsteroide(sdl::FPoint &pos) {
    int exp = SDL_floor(sdl::random(3)); // Elegir uno de los tres sonidos de explosión
    // Calcular angulo y distancia con respecto a centro de ventana
    float angulo = (SDL_atan2f(pos.Y()-240.0, pos.X()-320.0)+pi/2)/factor;
    while(angulo < 0) angulo+=360;
    float distancia = (pos.X()-320.0)*(pos.X()-320.0)+(pos.Y()-240.0)*(pos.Y()-240.0);
    distancia = 255.0*(distancia/(320.0*320.0));
    explosion[exp].Play();
    explosion[exp].SetPosition(static_cast<Sint16>(angulo), static_cast<Uint8>(distancia));
}

void Asteroides::RebotarAsteroides(int a, int b) {
    sdl::FPoint V1, V2;
    float theta;
    float m1,m2;

    m1 = (asteroides[a].Tipo()==grande) ? 4.0: (asteroides[a].Tipo()==mediano) ? 2.0:1.0;
    m2 = (asteroides[b].Tipo()==grande) ? 4.0: (asteroides[b].Tipo()==mediano) ? 2.0:1.0;
    // Calcular el ángulo normal:
    // θ = atan2(y2-y1, x2-x1)
    theta = SDL_atan2f(asteroides[b].Posicion().Y()-asteroides[a].Posicion().Y(),
                       asteroides[b].Posicion().X()-asteroides[a].Posicion().X());
    // Calcular componentes normal y transversal de las velocidades
    // Vn =  Vx · cos(θ) + Vy · sin(θ)
    // Vt = -Vx · sin(θ) + Vy · cos(θ)
    V1.X() =  asteroides[a].Velocidad().X()*cos(theta)+asteroides[a].Velocidad().Y()*sin(theta);
    V1.Y() = -asteroides[a].Velocidad().X()*sin(theta)+asteroides[a].Velocidad().Y()*cos(theta);
    V2.X() =  asteroides[b].Velocidad().X()*cos(theta)+asteroides[b].Velocidad().Y()*sin(theta);
    V2.Y() = -asteroides[b].Velocidad().X()*sin(theta)+asteroides[b].Velocidad().Y()*cos(theta);
    // Calcular nuevas direcciones
    // v1fn = (2·v2in·m2+v1in·(m1-m2))/(m1+m2)
    // v2fn = (2·v1in·m1+v2in·(m2-m1))/(m1+m2)
    // v1ft = v1it
    // v2ft = v2it
    float v2n = (2*V1.X()*m1+V2.X()*(m2-m1))/(m1+m2);
    float v1n = (2*V2.X()*m2+V1.X()*(m1-m2))/(m1+m2);
    V1.X() = v1n;
    V2.X() = v2n;
    // Recalcular componentes ortogonales:
    // Vx = Vn · cos(θ) - Vt · sin(θ)
    // Vy = Vn · sin(θ) + Vt · cos(θ)
    asteroides[a].Velocidad().X() = V1.X()*cos(theta)-V1.Y()*sin(theta);
    asteroides[a].Velocidad().Y() = V1.X()*sin(theta)+V1.Y()*cos(theta);
    asteroides[b].Velocidad().X() = V2.X()*cos(theta)-V2.Y()*sin(theta);
    asteroides[b].Velocidad().Y() = V2.X()*sin(theta)+V2.Y()*cos(theta);
}

void Asteroides::DividirAsteroide(int iAsteroide, float masa, sdl::FPoint &posicion, sdl::FPoint &velocidad) {
    sdl::FPoint V1, V2;
    float theta;
    float m1,m2;
    float v1n,v2n;
    sdl::FPoint p1, p2, p2b;
    tipoAsteroide tipo;

    if(asteroides[iAsteroide].Tipo()==pequeno) {
        asteroides[iAsteroide].Desactivar();
        return;
    }
    tipo = (asteroides[iAsteroide].Tipo()==grande) ? mediano:pequeno;
    m1 = (asteroides[iAsteroide].Tipo()==grande) ? 4.0:2.0;
    m2 = masa;
    // Calcular el ángulo normal:
    // θ = atan2(y2-y1, x2-x1)
    theta = SDL_atan2f(posicion.Y()-asteroides[iAsteroide].Posicion().Y(),
                       posicion.X()-asteroides[iAsteroide].Posicion().X());
    // Calcular componentes normal y transversal de las velocidades y posicion de asteroide
    // Vn =  Vx · cos(θ) + Vy · sin(θ)
    // Vt = -Vx · sin(θ) + Vy · cos(θ)
    V1.X() =  asteroides[iAsteroide].Velocidad().X()*cos(theta)+asteroides[iAsteroide].Velocidad().Y()*sin(theta);
    V1.Y() = -asteroides[iAsteroide].Velocidad().X()*sin(theta)+asteroides[iAsteroide].Velocidad().Y()*cos(theta);
    p1.X() =  asteroides[iAsteroide].Posicion().X()*cos(theta)+asteroides[iAsteroide].Posicion().Y()*sin(theta);
    p1.Y() = -asteroides[iAsteroide].Posicion().X()*sin(theta)+asteroides[iAsteroide].Posicion().Y()*cos(theta);
    V2.X() =  velocidad.X()*cos(theta)+velocidad.Y()*sin(theta);
    V2.Y() = -velocidad.X()*sin(theta)+velocidad.Y()*cos(theta);
    // Calcular nuevas direcciones
    // v1fn = (2·v2in·m2+v1in·(m1-m2))/(m1+m2)
    // v2fn = (2·v1in·m1+v2in·(m2-m1))/(m1+m2)
    // v1ft = v1it
    // v2ft = v2it
    v1n = v2n = (2*V1.X()*m1+V2.X()*(m2-m1))/(m1+m2);
    V1.X() = V2.X() = v1n;
    // Sumar velocidades de separación
    V1.Y() -= 100;
    V2.Y() += 100;
    // Trasladar las posiciones tangencialmente:
    p2 = p1;
    p1.Y() -= 15;
    p2.Y() += 15;
    // Recalcular componentes ortogonales de velocidad y posicion:
    // Vx = Vn · cos(θ) - Vt · sin(θ)
    // Vy = Vn · sin(θ) + Vt · cos(θ)
    asteroides[iAsteroide].Velocidad().X() = V1.X()*cos(theta)-V1.Y()*sin(theta);
    asteroides[iAsteroide].Velocidad().Y() = V1.X()*sin(theta)+V1.Y()*cos(theta);
    asteroides[iAsteroide].Posicion().X() = p1.X()*cos(theta)-p1.Y()*sin(theta);
    asteroides[iAsteroide].Posicion().Y() = p1.X()*sin(theta)+p1.Y()*cos(theta);
    p2b.X() = p2.X()*cos(theta)-p2.Y()*sin(theta);
    p2b.Y() = p2.X()*sin(theta)+p2.Y()*cos(theta);

    asteroides[iAsteroide].CambiarTipo(tipo);
    sdl::FPoint v2 = {V2.X()*SDL_cosf(theta)-V2.Y()*SDL_sinf(theta), V2.X()*SDL_sinf(theta)+V2.Y()*SDL_cosf(theta)};
    asteroides.push_back(Asteroide(p2b, tipo, v2));
    asteroides[iAsteroide].Actualizar(0, escala);
    asteroides[asteroides.size()-1].Actualizar(0, escala);
}

void Asteroides::Init() {
    // Añadir 5 asteroides grandes
    for(int i=0; i<5; i++) {
        asteroides.push_back(Asteroide({sdl::randomf(640.0), sdl::randomf(480.0)}, grande));
    }
}

int main( int argc, char * argv[] ) {
    sdl::SDL2 sdl(SDL_INIT_EVERYTHING);
    sdl::Window window ("Asteroides", 800, 600, (SDL_WindowFlags)(SDL_WINDOW_SHOWN | SDL_WINDOW_BORDERLESS));
    sdl::Renderer renderer(window, SDL_RENDERER_ACCELERATED);

    Asteroides asteroides(window, renderer);

    asteroides.Run();

    return 0;
}

