/*
 * Clases para manejar hilos
 */

#ifndef _SDL_THREAD
#define _SDL_THREAD

#include <SDL.h>
#include <cstdio>

namespace sdl {

class Thread {
protected:
    SDL_ThreadFunction tfunction;
    void* data;
    SDL_Thread *handle;
    int retval;
    bool detach;
public:
    Thread() : tfunction(NULL), data(NULL), handle(NULL), detach(false) {}
    Thread(SDL_ThreadFunction funct, const char* name, void* dat, const size_t stack=0) : tfunction(funct), data(dat), detach(false) {
        handle = SDL_CreateThreadWithStackSize(tfunction, name, stack, data);
    }
    ~Thread() {
        if(!detach) SDL_WaitThread(handle, &retval);
    }
    void SetFunction(SDL_ThreadFunction funct) { tfunction=funct; }
    void SetData(void *dat) { data = dat; }
    void Init(const char* name, const size_t stack=0) {
        if(tfunction != NULL) handle = SDL_CreateThreadWithStackSize(tfunction, name, stack, data);
    }
    int Wait() {
        if(!detach) {
            SDL_WaitThread(handle, &retval);
        } else {
            retval=-1;
        }
        return retval;
    }
    void Detach() {
        SDL_DetachThread(handle);
        detach = true;
    }
    const char* GetName() const {
        if(!detach) return SDL_GetThreadName(handle);
        return NULL;
    }
    SDL_threadID GetId() {
        if(!detach) return SDL_GetThreadID(handle);
        return 0;
    }
};

} // namespace

#endif // _SDL_THREAD

